@@ -1,5 +1,6 @@
 import os
 import re
+from subprocess import Popen, PIPE
 
 import requests
 import tarfile
@@ -7,6 +8,7 @@ import zipfile
 
 from selenium import webdriver
 from selenium.common.exceptions import SessionNotCreatedException
+from selenium.webdriver.remote.webdriver import WebDriver
 
 from pwdriver.val import CONFIG_NAME, CHROME, CHROMEDRIVER, CHROMEDRIVER_API, CHROMEDRIVER_NAME, DRIVER, \
     EDGE, EDGEDRIVER, EDGEDRIVER_API, EDGEDRIVER_NAME, GECKO, GECKODRIVER, GECKODRIVER_API, GECKODRIVER_NAME, \
@@ -17,26 +19,34 @@ logger = util.get_logger('core')
 driver_path = os.path.join(ROOT_DIR, DRIVER)
 
 
-def _get_local_chrome_version():
+def _get_local_chrome_version() -> str:
     if OS_NAME == 'WIN':
-        with os.popen(r'wmic datafile where name="C:\\Program Files\\Google\\Chrome\\Application\\chrome.exe" get '
-                      r'version /value') as stream:
-            version = stream.read().strip().strip('Version=')
-        if not version:
-            with os.popen(r'wmic datafile where name="C:\\Program Files ('
-                          r'x86)\\Google\\Chrome\\Application\\chrome.exe" get version /value') as stream:
-                version = stream.read().strip().strip('Version=')
+        path = ['Program Files', 'Program Files (x86)']
+        for loc in path:
+            with Popen(fr'wmic datafile where name="C:\\{loc}\\Google\\Chrome\\Application\\chrome.exe" get '
+                       r'version /value', stdout=PIPE, stderr=PIPE, shell=True) as p:
+                output, error = p.communicate()
+                if not error:
+                    version = output.decode('ansi').strip().strip('Version=')
+                    break
+                else:
+                    version = None
     elif OS_NAME == 'MAC':
-        with os.popen(r'/Applications/Google\ Chrome.app/Contents/MacOS/Google\ Chrome --version') as stream:
-            version = stream.read().strip('Google Chrome ').strip()
+        with Popen(r'/Applications/Google\ Chrome.app/Contents/MacOS/Google\ Chrome --version', stdout=PIPE,
+                   stderr=PIPE, shell=True) as p:
+            output, error = p.communicate()
+            version = output.decode('utf-8').strip().strip('Google Chrome ') if not error else None
     else:
-        with os.popen(r'google-chrome --version') as stream:
-            version = stream.read().strip('Google Chrome ').strip()
-    logger.info(f'Installed Chrome Browser version: {version}')
+        with Popen(r'google-chrome --version', stdout=PIPE, stderr=PIPE, shell=True) as p:
+            output, error = p.communicate()
+            version = output.decode('utf-8').strip().strip('Google Chrome ') if not error else None
+    logger.info(f'Installed {CHROME} Browser version: {version}')
+    if version is None:
+        raise NotImplementedError(f'{CHROME} browser not installed.')
     return version
 
 
-def _get_latest_chrome_version(version):
+def _get_latest_chrome_version(version) -> str:
     latest_release = requests.get(f'{CHROMEDRIVER_API}/LATEST_RELEASE_{re.split(r"[.]", version)[0]}')
     global driver_path
     if not driver_path.endswith(latest_release.text):
@@ -45,7 +55,7 @@ def _get_latest_chrome_version(version):
     return latest_release.text
 
 
-def _download_chromedriver(version):
+def _download_chromedriver(version) -> None:
     global driver_path
     if not os.path.isfile(os.path.join(driver_path, CHROMEDRIVER_NAME)):
         logger.info('Not found executable chromedriver. Chromedriver will be downloaded.')
@@ -67,14 +77,14 @@ def _download_chromedriver(version):
         logger.info(f'Executable driver found: ({os.path.abspath(driver_path)})')
 
 
-def setup_chromedriver():
+def setup_chromedriver() -> None:
     _download_chromedriver(_get_latest_chrome_version(_get_local_chrome_version()))
     global driver_path
     util.set_file_executable(os.path.join(driver_path, CHROMEDRIVER_NAME))
     os.environ['PATH'] += f'{os.pathsep}{os.path.abspath(driver_path)}'
 
 
-def _get_latest_gecko_version():
+def _get_latest_gecko_version() -> str:
     version = re.split(r'/+', requests.get(f'{GECKODRIVER_API}/latest', allow_redirects=True).url)[-1]
     global driver_path
     if not driver_path.endswith(version):
@@ -82,13 +92,14 @@ def _get_latest_gecko_version():
     return version
 
 
-def _download_geckodriver(version):
+def _download_geckodriver(version) -> None:
     global driver_path
     if not os.path.isfile(os.path.join(driver_path, GECKODRIVER_NAME)):
         logger.info(f'Not found executable geckodriver. Geckodriver will be downloaded.')
         download_url = f'{GECKODRIVER_API}/download/{version}/geckodriver-{version}-' \
                        + (('win64.zip' if OS_BIT == '64bit' else 'win32.zip') if OS_NAME == 'WIN'
-                          else 'macos.tar.gz' if OS_NAME == 'MAC' else 'linux' + ('64.tar.gz' if OS_BIT == '64bit' else '32.tar.gz'))
+                          else 'macos.tar.gz' if OS_NAME == 'MAC' else 'linux' + (
+                            '64.tar.gz' if OS_BIT == '64bit' else '32.tar.gz'))
         file = requests.get(download_url, stream=True)
         file_name = f'{GECKODRIVER}{ZIP}' if OS_NAME == 'WIN' else f'{GECKODRIVER}{TAR_GZ}'
         with open(file_name, 'wb') as fd:
@@ -110,33 +121,41 @@ def _download_geckodriver(version):
         logger.info(f'Executable driver found: ({os.path.abspath(driver_path)})')
 
 
-def setup_geckodriver():
+def setup_geckodriver() -> None:
     _download_geckodriver(_get_latest_gecko_version())
     global driver_path
     util.set_file_executable(os.path.join(driver_path, GECKODRIVER_NAME))
     os.environ['PATH'] += f'{os.pathsep}{os.path.abspath(driver_path)}'
 
 
-def _get_local_edge_version():
+def _get_local_edge_version() -> str:
     if OS_NAME == 'WIN':
-        with os.popen(r'wmic datafile where name="C:\\Program Files\\Microsoft\\Edge\\Application\\msedge.exe" get '
-                      r'version /value') as stream:
-            version = stream.read().strip().strip('Version=')
-        if not version:
-            with os.popen(r'wmic datafile where name="C:\\Program Files ('
-                          r'x86)\\Microsoft\\Edge\\Application\\msedge.exe" get version /value') as stream:
-                version = stream.read().strip().strip('Version=')
+        path = ['Program Files', 'Program Files (x86)']
+        for loc in path:
+            with Popen(fr'wmic datafile where name="C:\\{loc}\\Microsoft\\Edge\\Application\\msedge.exe" get '
+                       r'version /value', stdout=PIPE, stderr=PIPE, shell=True) as p:
+                output, error = p.communicate()
+                if not error:
+                    version = output.decode('ansi').strip().strip('Version=')
+                    break
+                else:
+                    version = None
     elif OS_NAME == 'MAC':
-        with os.popen(r'/Applications/Microsoft\ Edge.app/Contents/MacOS/Microsoft\ Edge --version') as stream:
-            version = stream.read().strip('Microsoft Edge ').strip()
+        with Popen(r'/Applications/Microsoft\ Edge.app/Contents/MacOS/Microsoft\ Edge --version', stdout=PIPE,
+                   stderr=PIPE, shell=True) as p:
+            output, error = p.communicate()
+            version = output.decode('utf-8').strip().strip('Microsoft Edge ') if not error else None
     else:
-        with os.popen(r'microsoft-edge --version') as stream:
-            version = stream.read().strip('Microsoft Edge ').strip()
-    logger.info(f'Installed Edge Browser version: {version}')
+        with Popen(r'microsoft-edge --version', stdout=PIPE, stderr=PIPE, shell=True) as p:
+            output, error = p.communicate()
+            version = output.decode('utf-8').strip().strip('Microsoft Edge ') if not error else None
+    logger.info(f'Installed {EDGE} browser version: {version}')
+    if version is None:
+        raise NotImplementedError(f'{EDGE} browser not installed.')
     return version
 
 
-def _download_edgedriver(version):
+def _download_edgedriver(version) -> None:
     global driver_path
     if not driver_path.endswith(version):
         driver_path = os.path.join(driver_path, EDGE, version)
@@ -161,7 +180,7 @@ def _download_edgedriver(version):
         logger.info(f'Executable driver found: ({os.path.abspath(driver_path)})')
 
 
-def setup_edgedriver():
+def setup_edgedriver() -> None:
     _download_edgedriver(_get_local_edge_version())
     global driver_path
     util.set_file_executable(os.path.join(driver_path, EDGEDRIVER_NAME))
@@ -186,7 +205,7 @@ class WebDriverFactory:
         if not self._automation_local:
             self._automation_url = config.get('automation', 'automation.url')
 
-    def launch(self, options=None):
+    def launch(self, options=None) -> WebDriver:
         try:
             if self._automation_browser == CHROME and self._automation_local:
                 setup_chromedriver()
